@@ -12,15 +12,19 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+import jinja2
 import json
 import logging
 import os
 import requests
+import six
 import tempfile as tf
+import yaml
 
 from heatclient.common import template_utils
 from mistral import context
 from mistral.workflow import utils as mistral_workflow_utils
+from swiftclient import exceptions as swiftexceptions
 
 from tripleo_common.actions import base
 from tripleo_common import constants
@@ -38,9 +42,7 @@ def _create_temp_file(data):
 
 
 class UploadTemplatesAction(base.TripleOAction):
-    """Upload default heat templates for TripleO.
-
-    """
+    """Upload default heat templates for TripleO."""
     def __init__(self, container=constants.DEFAULT_CONTAINER_NAME):
         super(UploadTemplatesAction, self).__init__()
         self.container = container
@@ -56,11 +58,63 @@ class UploadTemplatesAction(base.TripleOAction):
 
 
 class ProcessTemplatesAction(base.TripleOAction):
+    """Process Templates and Environments
+
+    This method processes the templates and files in a given deployment
+    plan into a format that can be passed to Heat.
+    """
 
     def __init__(self, container=constants.DEFAULT_CONTAINER_NAME):
         super(ProcessTemplatesAction, self).__init__()
         self.container = container
 
+    def _process_custom_roles(self):
+        swift = self._get_object_client()
+        try:
+            j2_role_file = swift.get_object(
+                self.container, constants.OVERCLOUD_J2_ROLES_NAME)[1]
+            role_data = yaml.safe_load(j2_role_file)
+        except swiftexceptions.ClientException:
+            LOG.info("No %s file found, skipping jinja templating"
+                     % constants.OVERCLOUD_J2_ROLES_NAME)
+            return
+        try:
+            # Iterate over all files in the plan container
+            # we j2 render any with the .j2.yaml suffix
+            container_files = swift.get_container(self.container)
+        except swiftexceptions.ClientException as ex:
+            error_msg = ("Error listing contents of container %s : %s"
+                         % (self.container, six.text_type(ex)))
+            LOG.error(error_msg)
+            raise Exception(error_msg)
+
+        for f in [f.get('name') for f in container_files[1]]:
+            # check to see if the file is a *.j2.yaml
+            # if it is, get it and template for roles
+            if f.endswith('.j2.yaml'):
+                LOG.info("jinja2 rendering %s" % f)
+                j2_template = swift.get_object(self.container, f)[1]
+
+                try:
+                    # Render the j2 template
+                    template = jinja2.Environment().from_string(j2_template)
+                    r_template = template.render(roles=role_data)
+                except jinja2.exceptions.TemplateError as ex:
+                    error_msg = ("Error rendering template %s : %s"
+                                 % (f, six.text_type(ex)))
+                    LOG.error(error_msg)
+                    raise Exception(error_msg)
+                try:
+                    # write the template back to the plan container
+                    yaml_f = f.replace('.j2.yaml', '.yaml')
+                    swift.put_object(
+                        self.container, yaml_f, r_template)
+                except swiftexceptions.ClientException as ex:
+                    error_msg = ("Error storing file %s in container %s"
+                                 % (yaml_f, self.container))
+                    LOG.error(error_msg)
+                    raise Exception(error_msg)
+
     def run(self):
         error_text = None
         ctx = context.ctx()
@@ -69,11 +123,23 @@ class ProcessTemplatesAction(base.TripleOAction):
         try:
             mistral_environment = mistral.environments.get(self.container)
         except Exception as mistral_err:
-            error_text = mistral_err.message
+            error_text = six.text_type(mistral_err)
             LOG.exception(
                 "Error retrieving Mistral Environment: %s" % self.container)
             return mistral_workflow_utils.Result(error=error_text)
 
+        try:
+            # if the jinja overcloud template exists, process it and write it
+            # back to the swift container before continuing processing.  The
+            # method called below should handle the case where the files are
+            # not found in swift, but if they are found and an exception
+            # occurs during processing, that exception will cause the
+            # ProcessTemplatesAction to return an error result.
+            self._process_custom_roles()
+        except Exception as err:
+            LOG.exception("Error occurred while processing custom roles.")
+            return mistral_workflow_utils.Result(error=six.text_type(err))
+
         template_name = mistral_environment.variables.get('template')
         environments = mistral_environment.variables.get('environments')
         env_paths = []
@@ -122,7 +188,7 @@ class ProcessTemplatesAction(base.TripleOAction):
                     object_request=_object_request))
 
         except Exception as err:
-            error_text = str(err)
+            error_text = six.text_type(err)
             LOG.exception("Error occurred while processing plan files.")
         finally:
             # cleanup any local temp files
