@@ -20,6 +20,7 @@ from mistral.workflow import utils as mistral_workflow_utils
 from oslo_concurrency.processutils import ProcessExecutionError
 
 from tripleo_common.actions import base
+from tripleo_common import constants
 from tripleo_common.utils import validations as utils
 
 
@@ -58,6 +59,36 @@ class GetPubkeyAction(base.TripleOAction):
         return public_key
 
 
+class Enabled(base.TripleOAction):
+    """Indicate whether the validations have been enabled."""
+
+    def __init__(self):
+        super(Enabled, self).__init__()
+
+    def _validations_enabled(self):
+        """Detect whether the validations are enabled on the undercloud."""
+        mistral = self._get_workflow_client()
+        try:
+            # NOTE: the `ssh_keys` environment is created by
+            # instack-undercloud only when the validations are enabled on the
+            # undercloud (or when they're installed manually). Therefore, we
+            # can check for its presence here:
+            mistral.environments.get('ssh_keys')
+            return True
+        except Exception:
+            return False
+
+    def run(self):
+        return_value = {'stderr': ''}
+        if self._validations_enabled():
+            return_value['stdout'] = 'Validations are enabled'
+            mistral_result = (return_value, None)
+        else:
+            return_value['stdout'] = 'Validations are disabled'
+            mistral_result = (None, return_value)
+        return mistral_workflow_utils.Result(*mistral_result)
+
+
 class ListValidationsAction(base.TripleOAction):
     """Return a set of TripleO validations"""
     def __init__(self, groups=None):
@@ -83,19 +114,22 @@ class ListGroupsAction(base.TripleOAction):
 
 class RunValidationAction(base.TripleOAction):
     """Run the given validation"""
-    def __init__(self, validation):
+    def __init__(self, validation, plan=constants.DEFAULT_CONTAINER_NAME):
         super(RunValidationAction, self).__init__()
         self.validation = validation
+        self.plan = plan
 
     def run(self):
         mc = self._get_workflow_client()
+        identity_file = None
         try:
             env = mc.environments.get('ssh_keys')
             private_key = env.variables['private_key']
             identity_file = utils.write_identity_file(private_key)
 
             stdout, stderr = utils.run_validation(self.validation,
-                                                  identity_file)
+                                                  identity_file,
+                                                  self.plan)
             return_value = {'stdout': stdout, 'stderr': stderr}
             mistral_result = (return_value, None)
         except ProcessExecutionError as e:
@@ -103,5 +137,6 @@ class RunValidationAction(base.TripleOAction):
             # Indicates to Mistral there was a failure
             mistral_result = (None, return_value)
         finally:
-            utils.cleanup_identity_file(identity_file)
+            if identity_file:
+                utils.cleanup_identity_file(identity_file)
         return mistral_workflow_utils.Result(*mistral_result)
