@@ -182,6 +182,66 @@ class FindNodeHandlerTest(base.TestCase):
                           nodes._find_node_handler, {'pm_type': 'foobar'})
 
 
+class NodeProvisionStateTest(base.TestCase):
+
+    def test_wait_for_provision_state(self):
+        baremetal_client = mock.Mock()
+        baremetal_client.node.get.return_value = mock.Mock(
+            provision_state="available", last_error=None)
+        nodes.wait_for_provision_state(baremetal_client, 'UUID', "available")
+
+    def test_wait_for_provision_state_not_found(self):
+        baremetal_client = mock.Mock()
+        baremetal_client.node.get.side_effect = exception.InvalidNode("boom")
+        self.assertRaises(
+            exception.InvalidNode,
+            nodes.wait_for_provision_state,
+            baremetal_client, 'UUID', "enroll")
+
+    def test_wait_for_provision_state_timeout(self):
+        baremetal_client = mock.Mock()
+        baremetal_client.node.get.return_value = mock.Mock(
+            provision_state="not what we want", last_error=None)
+        self.assertRaises(
+            exception.Timeout,
+            nodes.wait_for_provision_state,
+            baremetal_client, 'UUID', "available", loops=1, sleep=0.01)
+
+    def test_wait_for_provision_state_fail(self):
+        baremetal_client = mock.Mock()
+        baremetal_client.node.get.return_value = mock.Mock(
+            provision_state="enroll",
+            last_error="node on fire; returning to previous state.")
+        self.assertRaises(
+            exception.StateTransitionFailed,
+            nodes.wait_for_provision_state,
+            baremetal_client, 'UUID', "available", loops=1, sleep=0.01)
+
+    @mock.patch('tripleo_common.utils.nodes.wait_for_provision_state')
+    def test_set_nodes_state(self, wait_for_state_mock):
+
+        wait_for_state_mock.return_value = True
+        bm_client = mock.Mock()
+
+        # One node already deployed, one in the manageable state after
+        # introspection.
+        node_list = [
+            mock.Mock(uuid="ABCDEFGH", provision_state="active"),
+            mock.Mock(uuid="IJKLMNOP", provision_state="manageable")
+        ]
+
+        skipped_states = ('active', 'available')
+        affected_nodes = nodes.set_nodes_state(bm_client, node_list, 'provide',
+                                               'available', skipped_states)
+        uuids = [node.uuid for node in affected_nodes]
+
+        bm_client.node.set_provision_state.assert_has_calls([
+            mock.call('IJKLMNOP', 'provide'),
+        ])
+
+        self.assertEqual(uuids, ['IJKLMNOP', ])
+
+
 class NodesTest(base.TestCase):
 
     def _get_node(self):
@@ -279,6 +339,59 @@ class NodesTest(base.TestCase):
         ironic.port.create.assert_has_calls([port_call])
         ironic.node.set_power_state.assert_has_calls([power_off_call])
 
+    def test_register_all_nodes_uuid(self):
+        node_list = [self._get_node()]
+        node_list[0]['uuid'] = 'abcdef'
+        node_properties = {"cpus": "1",
+                           "memory_mb": "2048",
+                           "local_gb": "30",
+                           "cpu_arch": "amd64",
+                           "capabilities": "num_nics:6"}
+        ironic = mock.MagicMock()
+        nodes.register_all_nodes('servicehost', node_list, client=ironic)
+        pxe_node_driver_info = {"ssh_address": "foo.bar",
+                                "ssh_username": "test",
+                                "ssh_key_contents": "random",
+                                "ssh_virt_type": "virsh"}
+        pxe_node = mock.call(driver="pxe_ssh",
+                             name='node1',
+                             driver_info=pxe_node_driver_info,
+                             properties=node_properties,
+                             uuid="abcdef")
+        port_call = mock.call(node_uuid=ironic.node.create.return_value.uuid,
+                              address='aaa')
+        power_off_call = mock.call(ironic.node.create.return_value.uuid, 'off')
+        ironic.node.create.assert_has_calls([pxe_node, mock.ANY])
+        ironic.port.create.assert_has_calls([port_call])
+        ironic.node.set_power_state.assert_has_calls([power_off_call])
+
+    def test_register_all_nodes_caps_dict(self):
+        node_list = [self._get_node()]
+        node_list[0]['capabilities'] = {
+            'num_nics': 7
+        }
+        node_properties = {"cpus": "1",
+                           "memory_mb": "2048",
+                           "local_gb": "30",
+                           "cpu_arch": "amd64",
+                           "capabilities": "num_nics:7"}
+        ironic = mock.MagicMock()
+        nodes.register_all_nodes('servicehost', node_list, client=ironic)
+        pxe_node_driver_info = {"ssh_address": "foo.bar",
+                                "ssh_username": "test",
+                                "ssh_key_contents": "random",
+                                "ssh_virt_type": "virsh"}
+        pxe_node = mock.call(driver="pxe_ssh",
+                             name='node1',
+                             driver_info=pxe_node_driver_info,
+                             properties=node_properties)
+        port_call = mock.call(node_uuid=ironic.node.create.return_value.uuid,
+                              address='aaa')
+        power_off_call = mock.call(ironic.node.create.return_value.uuid, 'off')
+        ironic.node.create.assert_has_calls([pxe_node, mock.ANY])
+        ironic.port.create.assert_has_calls([port_call])
+        ironic.node.set_power_state.assert_has_calls([power_off_call])
+
     def test_register_update(self):
         node = self._get_node()
         ironic = mock.MagicMock()
@@ -308,6 +421,39 @@ class NodesTest(base.TestCase):
                                               client=ironic)
         ironic.node.update.assert_called_once_with(1, mock.ANY)
 
+    def test_register_update_with_images(self):
+        node = self._get_node()
+        node['kernel_id'] = 'image-k'
+        node['ramdisk_id'] = 'image-r'
+        ironic = mock.MagicMock()
+        node_map = {'mac': {'aaa': 1}}
+
+        def side_effect(*args, **kwargs):
+            update_patch = [
+                {'path': '/name', 'value': 'node1'},
+                {'path': '/driver_info/ssh_key_contents', 'value': 'random'},
+                {'path': '/driver_info/ssh_address', 'value': 'foo.bar'},
+                {'path': '/properties/memory_mb', 'value': '2048'},
+                {'path': '/properties/local_gb', 'value': '30'},
+                {'path': '/properties/cpu_arch', 'value': 'amd64'},
+                {'path': '/properties/cpus', 'value': '1'},
+                {'path': '/properties/capabilities', 'value': 'num_nics:6'},
+                {'path': '/driver_info/deploy_kernel', 'value': 'image-k'},
+                {'path': '/driver_info/deploy_ramdisk', 'value': 'image-r'},
+                {'path': '/driver_info/ssh_username', 'value': 'test'},
+                {'path': '/driver_info/ssh_virt_type', 'value': 'virsh'}]
+            for key in update_patch:
+                key['op'] = 'add'
+            self.assertThat(update_patch,
+                            matchers.MatchesSetwise(*(map(matchers.Equals,
+                                                          args[1]))))
+            return mock.Mock(uuid='uuid1')
+
+        ironic.node.update.side_effect = side_effect
+        nodes._update_or_register_ironic_node(None, node, node_map,
+                                              client=ironic)
+        ironic.node.update.assert_called_once_with(1, mock.ANY)
+
     def _update_by_type(self, pm_type):
         ironic = mock.MagicMock()
         node_map = {'mac': {}, 'pm_addr': {}}
@@ -361,6 +507,36 @@ class NodesTest(base.TestCase):
                                               client=ironic)
         ironic.node.update.assert_called_once_with(1, mock.ANY)
 
+    def test_register_node_update_with_uuid(self):
+        node = self._get_node()
+        node['uuid'] = 'abcdef'
+        ironic = mock.MagicMock()
+        node_map = {'uuids': {'abcdef'}}
+
+        def side_effect(*args, **kwargs):
+            update_patch = [
+                {'path': '/name', 'value': 'node1'},
+                {'path': '/driver_info/ssh_key_contents', 'value': 'random'},
+                {'path': '/driver_info/ssh_address', 'value': 'foo.bar'},
+                {'path': '/properties/memory_mb', 'value': '2048'},
+                {'path': '/properties/local_gb', 'value': '30'},
+                {'path': '/properties/cpu_arch', 'value': 'amd64'},
+                {'path': '/properties/cpus', 'value': '1'},
+                {'path': '/properties/capabilities', 'value': 'num_nics:6'},
+                {'path': '/driver_info/ssh_username', 'value': 'test'},
+                {'path': '/driver_info/ssh_virt_type', 'value': 'virsh'}]
+            for key in update_patch:
+                key['op'] = 'add'
+            self.assertThat(update_patch,
+                            matchers.MatchesSetwise(*(map(matchers.Equals,
+                                                          args[1]))))
+            return mock.Mock(uuid='abcdef')
+
+        ironic.node.update.side_effect = side_effect
+        nodes._update_or_register_ironic_node(None, node, node_map,
+                                              client=ironic)
+        ironic.node.update.assert_called_once_with('abcdef', mock.ANY)
+
     def test_register_ironic_node_int_values(self):
         node_properties = {"cpus": "1",
                            "memory_mb": "2048",
@@ -450,6 +626,13 @@ class NodesTest(base.TestCase):
                           nodes._get_node_id,
                           node, handler, node_map)
 
+    def test_get_node_id_valid_duplicate(self):
+        node = self._get_node()
+        handler = nodes._find_driver_handler('pxe_ipmitool')
+        node_map = {'mac': {'aaa': 'id'},
+                    'pm_addr': {'foo.bar': 'id'}}
+        self.assertEqual('id', nodes._get_node_id(node, handler, node_map))
+
 
 class TestPopulateNodeMapping(base.TestCase):
     def test_populate_node_mapping_ironic(self):
@@ -464,7 +647,8 @@ class TestPopulateNodeMapping(base.TestCase):
                                               [])
         client.node.list.return_value = [node1, node2]
         expected = {'mac': {'aaa': 'abcdef'},
-                    'pm_addr': {'10.0.1.2': 'fedcba'}}
+                    'pm_addr': {'10.0.1.2': 'fedcba'},
+                    'uuids': {'abcdef', 'fedcba'}}
         self.assertEqual(expected, nodes._populate_node_mapping(client))
 
     def test_populate_node_mapping_ironic_fake_pxe(self):
@@ -475,5 +659,6 @@ class TestPopulateNodeMapping(base.TestCase):
         node = ironic_node('abcdef', 'fake_pxe', None)
         client.node.list_ports.return_value = [ironic_port('aaa')]
         client.node.list.return_value = [node]
-        expected = {'mac': {'aaa': 'abcdef'}, 'pm_addr': {}}
+        expected = {'mac': {'aaa': 'abcdef'}, 'pm_addr': {},
+                    'uuids': {'abcdef'}}
         self.assertEqual(expected, nodes._populate_node_mapping(client))
